// kp shell
__global__ void kpshellsum(cuComplex* k2field2, int ikp, float* temp)
{
  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();
  if(Nz<=zThreads) {
  	if(idx<Nx && idy<Ny/2+1 && idz<Nz){ 
		if(sqrt(abs(kPerp2(idx,idy))) >= ikp-1.0 && 
		sqrt(abs(kPerp2(idx,idy))) < ikp+1.0)  {

			int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
			atomicAdd(temp + ikp, k2field2[index].x);
		}
	}
  }
  else{
		for(int i=0; i<Nz/zThreads; i++){
			if(idx<Nx && idy<Ny/2+1 && idz<zThreads){ 
				int IDZ = idz + zThreads*i;
				if(sqrt(abs(kPerp2(idx,idy))) >= ikp-1.0 && 
				sqrt(abs(kPerp2(idx,idy))) < ikp+1.0)  {

					int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*IDZ;
					atomicAdd(temp + ikp, k2field2[index].x);

				}
			}
		}
	}
}

__global__ void kpshellsum(float* k2field2, int ikp, float* temp)
{
  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();
  if(Nz<=zThreads) {
  	if(idx<Nx && idy<Ny/2+1 && idz<Nz){ 
		if(sqrt(abs(kPerp2(idx,idy))) >= ikp-1.0 && 
		sqrt(abs(kPerp2(idx,idy))) < ikp+1.0)  {

			int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
			atomicAdd(temp + ikp, k2field2[index]);
		}
	}
	}
	else{
		for(int i=0; i<Nz/zThreads; i++){
			if(idx<Nx && idy<Ny/2+1 && idz<zThreads){ 
				int IDZ = idz + zThreads*i;
				if(sqrt(abs(kPerp2(idx,idy))) >= ikp-1.0 && 
				sqrt(abs(kPerp2(idx,idy))) < ikp+1.0)  {

					int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*IDZ;
					atomicAdd(temp + ikp, k2field2[index]);

				}
			}
		}
	}

}
// kz-kp shell
__global__ void kz_kpshellsum(cuComplex* k2field2, int ikp, float* energy_kp)
{

  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();
  // X0 and Y0 needs to be 1 for this to make sense.
  int ikz_kp;

  if(Nz<=zThreads) {
  	if(idx<Nx && idy<Ny/2+1 && idz<Nz){
		if(sqrt(abs(kPerp2(idx,idy))) >= ikp-0.5 && sqrt(abs(kPerp2(idx,idy))) < ikp+0.5 )  {
			unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
			ikz_kp = idz + Nz*ikp;
			atomicAdd(energy_kp + ikz_kp, k2field2[index].x);
			}
		}
	}
  else {
  	for(int i=0; i<Nz/zThreads; i++) {
		if(idx<Nx && idy<Ny/2+1 && idz<zThreads) {
			if(sqrt(abs(kPerp2(idx,idy))) >= ikp-0.5 && sqrt(abs(kPerp2(idx,idy))) < ikp+0.5)  {
				unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz + Nx*(Ny/2+1)*zThreads*i;
				int IDZ = idz + zThreads*i;
				ikz_kp = IDZ + Nz*ikp;
				atomicAdd(energy_kp + ikz_kp, k2field2[index].x);
				}
			}
		}
	}
}
__global__ void kz_kpshellsum(float* k2field2, int ikp, float* energy_kp)
{

  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();
  // X0 and Y0 needs to be 1 for this to make sense.
  int ikz_kp;

  if(Nz<=zThreads) {
  	if(idx<Nx/2+1 && idz<Nz){
		if(sqrt(abs(kPerp2(idx,idy))) >= ikp-0.5 && sqrt(abs(kPerp2(idx,idy))) < ikp+0.5)  {
			unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
			ikz_kp = idz + Nz*ikp;
			atomicAdd(energy_kp + ikz_kp, k2field2[index]);
			}
		}
	}
  else {
  	for(int i=0; i<Nz/zThreads; i++) {
		if(idx<Nx/2+1 && idz<zThreads) {
			if(sqrt(abs(kPerp2(idx,idy))) >= ikp-0.5 && sqrt(abs(kPerp2(idx,idy))) < ikp+0.5)  {
				unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz + Nx*(Ny/2+1)*zThreads*i;
				int IDZ = idz + zThreads*i;
				ikz_kp = IDZ + Nz*ikp;
				atomicAdd(energy_kp + ikz_kp, k2field2[index]);
				}
			}
		}
	}
}

__global__ void fft_interp(float* result, cuComplex* function, float* xx, float* yy, float zz)
{

	unsigned int idx = get_idx();
	unsigned int idy = get_idy();
	unsigned int idz = get_idz();
    cuComplex cj; cj.x = 0.0f; cj.y = 1.0f;
	cuComplex tmp;
	if(Nz<=zThreads){
		if(idx<Nx && idy<Ny/2+1 && idz<Nz){
		  int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
		  tmp = function[index] * exp(cj*kx(idx)*xx[0] + cj*ky(idy)*yy[0] + cj*kz(idz)*zz);
		  if(idy!=0) tmp = tmp*2.0f;
		  atomicAdd(result, tmp.x);
		  }
		 }
	else{
		for(int i=0; i<Nz/zThreads; i++){
			if(idx<Nx && idy<Ny/2+1 && idz<zThreads){
				int IDZ = idz + zThreads*i;
				int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*IDZ;
				tmp = function[index] * exp(cj*kx(idx)*xx[0] + cj*ky(idy)*yy[0] + cj*kz(IDZ)*zz);
				if(idy!=0) tmp = tmp*2.0f;
			    atomicAdd(result, tmp.x);
				}
			}
		}
}


__global__ void fldtracestep(float* cnew, float* cold,  float* dB, float dz)
{
		cnew[0] = cold[0] + dB[0]*dz;
		cnew[0] = cnew[0] - 2.0*M_PI*((float) floor(cnew[0]/(2.0*M_PI) + 0.0*0.5));
}

__global__ void assign_fld(float* xfld, float* yfld, float *xfld0, float *yfld0)
{
	xfld[0] = xfld0[0];
	yfld[0] = yfld0[0];
}

__global__ void assign_foot(float* xfld0, float* yfld0)
{
    unsigned int idx = get_idx();
	if(idx<Nz){
		xfld0[idx] = 0.0;
		yfld0[idx] = 1.0;
	}
}

__global__ void sl_k_damp_calc(float *Gmdamp, cuComplex* Gm, float nu_kp_g, int alpha_kp_g, float nu_kz_g, int alpha_kz_g, int m){
  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();

  if(Nz<=zThreads) {
    if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
      unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
	  int idxmax = (Nx-1)/3; int idymax = (Ny-1)/3; int idzmax = (Nz-1)/3;

		Gmdamp[index] = (Gm[index].x*Gm[index].x + Gm[index].y*Gm[index].y)
					*(1.0f - exp(-2.0f*nu_kp_g*pow(abs(kPerp2(idx,idy)/kPerp2(idxmax,idymax)),alpha_kp_g))
					*exp(-2.0f*nu_kz_g*pow(abs(kz(idz)/kz(idzmax)),2*alpha_kz_g)));
		}
		}
  else {
    for(int i=0; i<Nz/zThreads; i++) {
      if(idy<(Ny/2+1) && idx<Nx && idz<zThreads) {
        unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz + Nx*(Ny/2+1)*zThreads*i;
	    int idxmax = (Nx-1)/3; int idymax = (Ny-1)/3; int idzmax = (Nz-1)/3;
		int IDZ = idz + zThreads*i;

		Gmdamp[index] = (Gm[index].x*Gm[index].x + Gm[index].y*Gm[index].y)
					 *(1.0f - exp(-2.0f*nu_kp_g*pow(abs(kPerp2(idx,idy)/kPerp2(idxmax,idymax)),alpha_kp_g))
					 *exp(-2.0f*nu_kz_g*pow(abs(kz(IDZ)/kz(idzmax)),2*alpha_kz_g)));
			}
		}
	}

}

__global__ void sl_tot_damp_calc(float *Gmdamp, cuComplex* Gm, float nu_kp_g, int alpha_kp_g, float nu_kz_g, int alpha_kz_g, float nu_coll, int alpha_m, int m){
  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();

  if(Nz<=zThreads) {
    if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
      unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
	  int idxmax = (Nx-1)/3; int idymax = (Ny-1)/3; int idzmax = (Nz-1)/3;

		if(m>=3) {Gmdamp[index] += (Gm[index].x*Gm[index].x + Gm[index].y*Gm[index].y)
					*(1.0f -
					exp(-2.0f*nu_kp_g*pow(abs(kPerp2(idx,idy)/kPerp2(idxmax,idymax)),alpha_kp_g))
					*exp(-2.0f*nu_kz_g*pow(abs(kz(idz)/kz(idzmax)),2*alpha_kz_g))
					*exp(-2.0f*nu_coll *pow(((float) m/(Nm-1)),2*alpha_m)));
				  }
		else {Gmdamp[index] += (Gm[index].x*Gm[index].x + Gm[index].y*Gm[index].y)
					*(1.0f - exp(-2.0f*nu_kp_g*pow(abs(kPerp2(idx,idy)/kPerp2(idxmax,idymax)),alpha_kp_g))
					*exp(-2.0f*nu_kz_g*pow(abs(kz(idz)/kz(idzmax)),2*alpha_kz_g)));
				  }
		}
	}
  else {
    for(int i=0; i<Nz/zThreads; i++) {
      if(idy<(Ny/2+1) && idx<Nx && idz<zThreads) {
        unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz + Nx*(Ny/2+1)*zThreads*i;
	    int idxmax = (Nx-1)/3; int idymax = (Ny-1)/3; int idzmax = (Nz-1)/3;
	    int IDZ = idz + zThreads*i;

		if(m>=3) {Gmdamp[index] += (Gm[index].x*Gm[index].x + Gm[index].y*Gm[index].y)
					*(1.0f -
					exp(-2.0f*nu_kp_g*pow(abs(kPerp2(idx,idy)/kPerp2(idxmax,idymax)),alpha_kp_g))
					*exp(-2.0f*nu_kz_g*pow(abs(kz(IDZ)/kz(idzmax)),2*alpha_kz_g))
					*exp(-2.0f*nu_coll *pow(((float) m/(Nm-1)),2*alpha_m)));
				}
		else{Gmdamp[index] += (Gm[index].x*Gm[index].x + Gm[index].y*Gm[index].y)
					*(1.0f - exp(-2.0f*nu_kp_g*pow(abs(kPerp2(idx,idy)/kPerp2(idxmax,idymax)),alpha_kp_g))
					*exp(-2.0f*nu_kz_g*pow(abs(kz(IDZ)/kz(idzmax)),2*alpha_kz_g)));
				}
			}
		}
	}
}


// Converts Gm to Gm+ or Gm- as defined by Alex
__global__ void Gm_to_Gtilde_p(cuComplex* Gm, cuComplex* gmtilde, int m){

  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();

  cuComplex cj;
  cj.x = 0.0f; cj.y = 1.0f;
  if(Nz<=zThreads) {
  	if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
		unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*idz;
		unsigned int index_next = index + Nx*(Ny/2+1)*Nz;

		gmtilde[index] = pow(cj*sgn(kz(idz)),m)* \
					(Gm[index] + cj*sgn(kz(idz))*Gm[index_next])/2.0;

		}
	}
  else {
  	for(int i=0; i<Nz/zThreads; i++) {
		if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
			int IDZ = idz + zThreads*i;
			unsigned int index = idy + (Ny/2+1)*idx + Nx*(Ny/2+1)*IDZ;
			unsigned int index_next = index + Nx*(Ny/2+1)*Nz;

				gmtilde[index] = pow(cj*sgn(kz(IDZ)),m)* \
							(Gm[index] + cj*sgn(kz(IDZ))*Gm[index_next])/2.0;
		}
	}
	}
}

// Converts Gm to Gm+ or Gm- as defined by Alex
__global__ void Gm_to_Gtilde_m(cuComplex* Gm, cuComplex* gmtilde, int m){

  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();

  cuComplex cj;
  cj.x = 0.0f; cj.y = 1.0f;
  if(Nz<=zThreads) {
  	if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
		unsigned int index = idy + (Ny/2+1)*idx+Nx*(Ny/2+1)*idz;
		unsigned int index_next = index + Nx*(Ny/2+1)*Nz;

		gmtilde[index] = pow(-1.,m)* pow(cj*sgn(kz(idz)),m)* \
					(Gm[index] - cj*sgn(kz(idz))*Gm[index_next])/2.0;

		}
	}
  else {
  	for(int i=0; i<Nz/zThreads; i++) {
		if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
			int IDZ = idz + zThreads*i;
			unsigned int index = idy + (Ny/2+1)*idx+Nx*(Ny/2+1)*IDZ;
			unsigned int index_next = index + Nx*(Ny/2+1)*Nz;

				gmtilde[index] = pow(-1.,m)* pow(cj*sgn(kz(IDZ)),m)* \
							(Gm[index] - cj*sgn(kz(IDZ))*Gm[index_next])/2.0;
		}
	}
	}
}
// Calculates exact flux in m
__global__ void m_flux(cuComplex* Gm, cuComplex* flux, int m){

  unsigned int idx = get_idx();
  unsigned int idy = get_idy();
  unsigned int idz = get_idz();
  cuComplex cj; cj.x = 0.0f; cj.y = 1.0f;

  if(Nz<=zThreads) {
  	if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
		unsigned int index = idy + (Ny/2+1)*idx+Nx*(Ny/2+1)*idz;
		unsigned int index_next = index + Nx*(Ny/2+1)*Nz;

		flux[index] = cj*kz(idz) *sqrt(2.*(m+1)) *(Gm[index_next]*conjg(Gm[index]));

		}
	}
  else {
  	for(int i=0; i<Nz/zThreads; i++) {
		if(idy<(Ny/2+1) && idx<Nx && idz<Nz) {
			int IDZ = idz + zThreads*i;
			unsigned int index = idy + (Ny/2+1)*idx+Nx*(Ny/2+1)*IDZ;
			unsigned int index_next = index + Nx*(Ny/2+1)*Nz;

			flux[index] = cj*kz(idz) *sqrt(2.*(m+1)) *(Gm[index_next]*conjg(Gm[index]));
		}
	}
	}
}
