//////////////////////////////////////////////////////////////////////
	// Alfven collision diagnostic
//////////////////////////////////////////////////////////////////////
    void aw_coll_diag(cuComplex* zp2, cuComplex* zm2, float time){

        int ikx, iky, ikz, index;

        cuComplex *energy;
        energy = (cuComplex*) malloc(sizeof(cuComplex)*2); 

        // Primary mode (1, 0, -1) copied over to energy[0]
        ikx = 1; iky=0; ikz = Nz-1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy, zp2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Primary mode (0, 1, 1) copied over to energy[1]
        ikx = 0; iky=1; ikz = 1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy+1, zp2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Primary modes written out. The ky=0 mode gets scaled up due to reality condition
        fprintf(awp_collfile, "%g \t %g \t", time, sqrt(energy[0].x+energy[1].x));

        // Secondary mode (1, 1, 0) copied over to energy[0]
        ikx = 1; iky=1; ikz = 0;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy, zp2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        //Secondary mode written out
        fprintf(awp_collfile, "%g \t ", sqrt(energy[0].x));

        // Tertiary mode (2, 1, -1) copied over to energy[0]
        ikx = 2; iky=1; ikz = Nz-1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy, zp2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Tertiary mode (1, 2, 1) copied over to energy[1]
        ikx = 1; iky=2; ikz = 1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy+1, zp2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Tertiary mode written out
        fprintf(awp_collfile, "%g\n", sqrt(energy[0].x + energy[1].x));

        fflush(awp_collfile);

        // Primary mode (1, 0, -1) copied over to energy[0]
        ikx = 1; iky=0; ikz = Nz-1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy, zm2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Primary mode (0, 1, 1) copied over to energy[1]
        ikx = 0; iky=1; ikz = 1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy+1, zm2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Primary modes written out. The ky=0 mode gets scaled up due to reality condition
        fprintf(awm_collfile, "%g \t %g \t", time, sqrt(energy[0].x+energy[1].x));

        // Secondary mode (1, 1, 0) copied over to energy[0]
        ikx = 1; iky=1; ikz = 0;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy, zm2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        //Secondary mode written out
        fprintf(awm_collfile, "%g \t ", sqrt(energy[0].x));

        // Tertiary mode (2, 1, -1) copied over to energy[0]
        ikx = 2; iky=1; ikz = Nz-1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy, zm2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Tertiary mode (1, 2, 1) copied over to energy[1]
        ikx = 1; iky=2; ikz = 1;
        index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
        cudaMemcpy(energy+1, zm2 + index, sizeof(cuComplex), cudaMemcpyDeviceToHost);

        // Tertiary mode written out
        fprintf(awm_collfile, "%g\n", sqrt(energy[0].x + energy[1].x));

        fflush(awm_collfile);

        free(energy);


    }
//////////////////////////////////////////////////////////////////////
	// Alfven diagnostics
//////////////////////////////////////////////////////////////////////

// kz- Kperp spectra of Alfven waves

	void energy_kz_kperp(cuComplex* kPhi, cuComplex* kA, float time)
	{
		// AVK: This could be improved upon
		int ikpmax = (int) ceil( sqrt( pow((float)(Nx-1)/3, 2) + pow((float)(Ny-1)/3, 2) ) );
		float kpmax = ceil( sqrt( pow((float)(Nx-1)/3, 2) + pow((float)(Ny-1)/3, 2) ) );
		float *kinEnergy_kp, *magEnergy_kp;

		cudaMalloc((void**) &kinEnergy_kp, sizeof(float)*ikpmax*Nz);
		cudaMalloc((void**) &magEnergy_kp, sizeof(float)*ikpmax*Nz);

		zero<<<Nz,ikpmax>>>(kinEnergy_kp, Nz*ikpmax,1,1);
		zero<<<Nz,ikpmax>>>(magEnergy_kp, Nz*ikpmax,1,1);

		float *kinEnergy_kp_h, *magEnergy_kp_h;

		kinEnergy_kp_h = (float*) malloc(sizeof(float)*ikpmax*Nz);
		magEnergy_kp_h = (float*) malloc(sizeof(float)*ikpmax*Nz);
		
			//loop through the ky's
			for(int ikp=1; ikp<ikpmax; ikp++) {
				kz_kpshellsum<<<dimGrid, dimBlock>>>(kPhi, ikp, kinEnergy_kp);
				kz_kpshellsum<<<dimGrid, dimBlock>>>(kA, ikp, magEnergy_kp);
			}

			if(debug) {printf("kz_kpshellsum: %s\n",cudaGetErrorString(cudaGetLastError())); }

			cudaMemcpy(kinEnergy_kp_h, kinEnergy_kp, sizeof(float)*ikpmax*Nz, cudaMemcpyDeviceToHost);
			cudaMemcpy(magEnergy_kp_h, magEnergy_kp, sizeof(float)*ikpmax*Nz, cudaMemcpyDeviceToHost);

			if(debug) { printf("Copying shell sums ME: %s\n",cudaGetErrorString(cudaGetLastError())); }

		for(int ikz=0; ikz<=(Nz-1)/3; ikz++){
			for(int ikp =1; ikp<ikpmax; ikp++){
				int ikz_kp = ikz + Nz*ikp;
				int mikz_kp = (Nz-ikz) + Nz*ikp;
				fprintf(alf_kzkpfile, "%g \t %g \t %g \t %1.12e \t %1.12e\n", time, kz(ikz), ((float) ikp/ikpmax)*kpmax, kinEnergy_kp_h[ikz_kp] + kinEnergy_kp_h[mikz_kp], magEnergy_kp_h[ikz_kp] + magEnergy_kp_h[mikz_kp]);
			}

		fprintf(alf_kzkpfile, "\n");
			}
		fprintf(alf_kzkpfile, "\n");
		
		cudaFree(kinEnergy_kp); cudaFree(magEnergy_kp);
		free(kinEnergy_kp_h); free(magEnergy_kp_h);
		
		
	}    

////////////////////////////////////////
// Total energy
	void energy(cuComplex* kPhi, cuComplex* kA, float time)
	{
	  if(debug) {printf("Entering energy\n");}

	  cuComplex *padded;
	  cudaMalloc((void**) &padded, sizeof(cuComplex)*Nx*Ny*Nz);
		
		
	  cuComplex *totEnergy_h, *kinEnergy_h, *magEnergy_h;

	  totEnergy_h = (cuComplex*) malloc(sizeof(cuComplex));
	  kinEnergy_h = (cuComplex*) malloc(sizeof(cuComplex));
	  magEnergy_h = (cuComplex*) malloc(sizeof(cuComplex));

	  // integrate kPhi to find kinetic energy
	  //sumReduc(kinEnergy_h, kPhi, padded);
	  sumReduc_gen(kinEnergy_h, kPhi, padded, Nx, Ny, Nz);
	  if(debug){ printf("sumreduc kPhi: %s\n",cudaGetErrorString(cudaGetLastError())); }
		
	  // integrate kA to find magnetic energy
	  //sumReduc(magEnergy_h, kA, padded);
	  sumReduc_gen(magEnergy_h, kA, padded, Nx, Ny, Nz);
	  if(debug){ printf("sumreduc kA: %s\n",cudaGetErrorString(cudaGetLastError())); }
		
	  //calculate total energy
	  totEnergy_h[0].x = kinEnergy_h[0].x + magEnergy_h[0].x;
		
	  cudaFree(padded);

	  fprintf(energyfile, "\t%g\t%g\t%g\t%g\n", time, totEnergy_h[0].x, kinEnergy_h[0].x, magEnergy_h[0].x);
	  printf("Total Energy = %g\t Kin Energy = %g\t Magnetic Energy = %g\n", totEnergy_h[0].x, kinEnergy_h[0].x, magEnergy_h[0].x);

	  free(totEnergy_h); free(kinEnergy_h); free(magEnergy_h); 
		
	  if(debug) {printf("Exiting energy\n");}
		
	}    

//////////////////////////////////////////////////////////////////////
	// Main diagnostic calling functions
//////////////////////////////////////////////////////////////////////

void alf_diagnostics(cuComplex* kPhi, cuComplex* kA, cuComplex* zp, cuComplex* zm, float time){
		// Calculate kperp**2 * phi and kperp**2 A for all alfven diagnostics
		addsubt<<<dimGrid,dimBlock>>> (kPhi, zp, zm, 1);
		//kPhi = zp+zm
			
		scale<<<dimGrid,dimBlock>>> (kPhi, .5);
		//kPhi = .5*(zp+zm) = phi
		
		addsubt<<<dimGrid,dimBlock>>> (kA, zp, zm, -1);
		//kA = zp-zm
		
		scale<<<dimGrid,dimBlock>>> (kA, .5);
		//kA = .5*(zp-zm) = A

		squareComplex<<<dimGrid,dimBlock>>> (kPhi);
		//kPhi = phi**2
		
		squareComplex<<<dimGrid,dimBlock>>> (kA);
		//kA = A**2

		fixFFT<<<dimGrid,dimBlock>>>(kPhi);
		fixFFT<<<dimGrid,dimBlock>>>(kA);
		
		multKPerp<<<dimGrid,dimBlock>>> (kPhi, kPhi, -1);
		//kPhi = (kperp**2) * (phi**2)

		multKPerp<<<dimGrid,dimBlock>>> (kA, kA, -1);
		//kA = (kperp**2) * (A**2)

		energy_kz_kperp(kPhi, kA, time);
		fflush(alf_kzkpfile);

		energy(kPhi, kA, time);
		fflush(energyfile);

        // Alfven wave collision diagnostics
        cudaMemcpy(kPhi, zp, sizeof(cuComplex)*Nx*(Ny/2+1)*Nz, cudaMemcpyDeviceToDevice);
        cudaMemcpy(kA, zm, sizeof(cuComplex)*Nx*(Ny/2+1)*Nz, cudaMemcpyDeviceToDevice);

		squareComplex<<<dimGrid,dimBlock>>> (kPhi);
		//kPhi = z+^2
		
		squareComplex<<<dimGrid,dimBlock>>> (kA);
		//kA = z-^2

        // No fixfft needed. If you called fixfft here, then the ky=0 mode needs to be
        // dealt with specially in the aw_coll_diag subroutine 
        aw_coll_diag(kPhi, kA, time);

}

//////////////////////////////////////////////////////////////////////
// Initialize diagnostics
//////////////////////////////////////////////////////////////////////
void init_diag(){

	  char str[255];

      /////////////////////////////////////////////////////////////////
      //Alfven diagnostics
      /////////////////////////////////////////////////////////////////
	  strcpy(str, runname);
	  strcat(str, ".energy");
      energyfile = fopen( str, "w+");
	  strcpy(str, runname);
	  strcat(str, ".speckzkp");
      alf_kzkpfile = fopen( str, "w+");

      /////////////////////////////////////////////////////////////////
      //Alfven wave collision diagnostic
      /////////////////////////////////////////////////////////////////
	  strcpy(str, runname);
	  strcat(str, ".awcollp");
      awp_collfile = fopen( str, "w+");

	  strcpy(str, runname);
	  strcat(str, ".awcollm");
      awm_collfile = fopen( str, "w+");
      
      // Initialize frequency diagnostic for NetCDF output
      printf("Initial ncid value: %d \n",ncid);
      init_freq_diag();
      printf("Chosen ncid value: %d \n",ncid);

}
//////////////////////////////////////////////////////////////////////
void close_diag(){

	fclose(energyfile);
	fclose(alf_kzkpfile);

    fclose(awp_collfile);
    fclose(awm_collfile);
    
    finish_freq_diag(ncid);

}

//////////////////////////////////////////////////////////////////////
void diagnostics(cuComplex* zp, cuComplex* zm, float time, int nout){
    alf_diagnostics(temp1, temp2, zp, zm, time);
    freq_diag(ncid, zp, zm, time, nout);
}
