//////////////////////////////////////////////////////////////////////
	// Frequency Diagnostic for NetCDF Movie Output
//////////////////////////////////////////////////////////////////////
// NOTE: The NetCDF file is formatted to be similar to the file structure
//       in the frequency diagnostics of AstroGK
//////////////////////////////////////////////////////////////////////


void freq_diag(int ncid, cuComplex* zp_d, cuComplex* zm_d, float time, int nout){
  int retval;
  int ikx, iky, ikz, index;
  int inkx,inky,inkz;
  cuComplex *zp_h;
  cuComplex *zm_h;
  float* zp_out;
  float* zm_out;
  int indout;
  float zpri[2];
  float zmri[2];
  int nkx,nky,nkz; //Number of Fourier modes to keep after dealiasing
  int nk_freq;
  bool dbg=true;
  size_t start[4];
  size_t count[4];
  size_t st1[1];
  size_t co1[1];
  

  // Function declarations
  int inky_to_iky(int);
  int inkx_to_ikx(int);
  int inkz_to_ikz(int);

   // Handle errors by printing an error message and exiting with a
  // non-zero status.
#define ERRCODE 2
#define ERR(e) {printf("Error: %s\n", nc_strerror(e)); exit(ERRCODE);}


  nkx=2*(Nx-1)/3+1;
  nky=(Ny-1)/3+1;
  nkz=2*(Nz-1)/3+1;
  nk_freq=nkx*nky;


  // Copy zp and zm data from Device
  zp_h = (cuComplex*) malloc(sizeof(cuComplex)*Nx*(Ny/2+1)*Nz);
  cudaMemcpy(zp_h, zp_d, sizeof(cuComplex)*Nx*(Ny/2+1)*Nz, cudaMemcpyDeviceToHost);
  if(debug) printf("Copying over zp %s\n",cudaGetErrorString(cudaGetLastError()));
  zm_h = (cuComplex*) malloc(sizeof(cuComplex)*Nx*(Ny/2+1)*Nz);
  cudaMemcpy(zm_h, zm_d, sizeof(cuComplex)*Nx*(Ny/2+1)*Nz, cudaMemcpyDeviceToHost);

  //Reduce data to nonzero Fourier modes (fully dealiased)
  // Allocate reduced arrays for output zp_out and zm_out
  zp_out=(float*) malloc(sizeof(float)*nk_freq*nkz*2);
  zm_out=(float*) malloc(sizeof(float)*nk_freq*nkz*2);

  if (dbg && 0==1) {
    for (ikz = 0; ikz < Nz;  ikz++){
      for (ikx = 0; ikx < Nx; ikx++){
	for (iky = 0; iky < (Ny/2+1); iky++){
	  index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;
 	  if (pow(zp_h[index].x,2)+pow(zp_h[index].y,2) > 1.0E-08 || pow(zm_h[index].x,2)+pow(zm_h[index].y,2) > 1.0E-08) 	    printf("index= %6d  iky= %3d  ikx= %3d  ikz= %3d ||  zp= %4g, %4g  zm= %4g, %4g\n",index,iky,ikx,ikz,zp_h[index].x,zp_h[index].y,zm_h[index].x,zm_h[index].y);
	    
	  
	}
      }
    }
  }

  //Copy full data into reduced complex Fourier fields for output
  for (inky = 0; inky < nky; inky++){
    for (inkx = 0; inkx < nkx; inkx++){
      for (inkz = 0; inkz < nkz; inkz++){
	
 	// Compute ikx,iky,ikz from inkx,inky,inkz
	// GGH NOTE: This is hokey and needs to be cleaned up, but it works
	iky=inky_to_iky(inky);
	ikx=inkx_to_ikx(inkx);
	ikz=inkz_to_ikz(inkz);
	if (ikx<0) ikx=Nx+ikx;
	if (ikz<0) ikz=Nz+ikz;
	
	
	// Get Index for zp_h and zm_h full complex data
	index = iky+(Ny/2+1)*ikx + (Ny/2+1)*Nx*ikz;

	// Construct 1D index for zp_out and zm_out
	//    NOTE: Factor of two is for real/imaginary
	indout=2*(inky*nkx*nkz + inkx*nkz + inkz);
	zp_out[indout]=zp_h[index].x;
	zp_out[indout+1]=zp_h[index].y;
	zm_out[indout]=zm_h[index].x;
	zm_out[indout+1]=zm_h[index].y;

	if (dbg && 0==1) {
	  //	  if (pow(zp_out[indout],2)+pow(zp_out[indout+1],2) > 1.0E-08 || pow(zm_out[indout],2)+pow(zm_out[indout],2) > 1.0E-08) 	  printf("indout= %6d index= %6d  inky= %3d  inkx= %3d  inkz= %3d  iky= %3d  ikx= %3d  ikz= %3d   ||  zp= %4g, %4g   zm= %4g, %4g\n",indout,index,inky,inkx,inkz,iky,ikx,ikz,zp_out[indout],zp_out[indout+1],zm_out[indout],zm_out[indout+1]);
	  printf("indout= %6d index= %6d  inky= %3d  inkx= %3d  inkz= %3d  iky= %3d  ikx= %3d  ikz= %3d   ||  zp= %4g, %4g   zm= %4g, %4g\n",indout,index,inky,inkx,inkz,iky,ikx,ikz,zp_out[indout],zp_out[indout+1],zm_out[indout],zm_out[indout+1]);
  }
	
      }
    }
  }

  // Write Fields for this timestep to NetCDF file
  // Write time for output
  st1[0]=nout;
  co1[0]=1;
  if ((retval = nc_put_vara_float(ncid, time_freq_id, st1, co1, &time)))
    ERR(retval);
  // Write zp and zm
  start[0]=nout;
  start[1]=0;
  start[2]=0;
  start[3]=0;
  count[0]=1;
  count[1]=nk_freq;
  count[2]=nkz;
  count[3]=2;
  if ((retval = nc_put_vara_float(ncid, zp_freq_id, start, count, zp_out)))
    ERR(retval);
  if ((retval = nc_put_vara_float(ncid, zm_freq_id, start, count, zm_out)))
    ERR(retval);
  
   //Sync to write to file
   if ((retval = nc_sync(ncid)))
     ERR(retval);
  
}

//////////////////////////////////////////////////////////////////////
// Create NetCDF file and define all dimensions and variables
void init_freq_diag(){
  
  // NOTE: ncid, time_freq_id, zp_freq_id, zm_freq_id are GLOBAL (declared in gandalf.cu)
  int retval;
  int i;
  int inkx,inky,inkz;
  int ikykx; //Index for 2D array into row-major 1D array
  char filename[255];
  int nkx,nky,nkz; //Number of Fourier modes to keep after dealiasing
  int nk_freq;
  int nakyx_freq_dim, kykx_freq_dim, kz_freq_dim, time_freq_dim, ri_freq_dim;
  // int* time_freq_dim,kz_freq_dim;
  //  int time_freq_id, akyx_freq_id, kz_freq_id, zp_freq_id, zm_freq_id;
  int akyx_freq_id, kz_freq_id;
  int kykx_mode_dim[2];
  int freq_mode_dim[4];
  float *kykx;
  float *allkz;
  int *ikyx_mode;
  char title[31]="Gandalf simulation fields data";
  bool dbg=true;

  // Function declarations
  int inky_to_iky(int);
  int inkx_to_ikx(int);
  int inkz_to_ikz(int);
  void create_2D_float_array(float **, int, int);
  void destroy_2D_float_array(float **, int, int);

  nkx=2*(Nx-1)/3+1;
  nky=(Ny-1)/3+1;
  nkz=2*(Nz-1)/3+1;

  nk_freq=nkx*nky;

  if (dbg) printf("Computed data dimensions \n");
  
  //#define title "Gandalf simulation fields data"

  // Handle errors by printing an error message and exiting with a
  // non-zero status.
#define ERRCODE 2
#define ERR(e) {printf("Error: %s\n", nc_strerror(e)); exit(ERRCODE);}


  // Create NetCDF output filename
  strcpy(filename, runname);
  strcat(filename, ".freq.nc");

  // Create NetCDF file 
  if ((retval = nc_create(filename, NC_CLOBBER, &ncid)))
    ERR(retval);

  printf("NetCDF File ID ncid= %d \n",ncid);

  if (dbg) printf("Created NetCDF file %s \n",filename);
  

  //Define Dimensions
  if ((retval = nc_def_dim(ncid, "kykx_modes", nk_freq, &nakyx_freq_dim)))
    ERR(retval);
  if ((retval = nc_def_dim(ncid, "kykx_dim", 2, &kykx_freq_dim)))
    ERR(retval);
  if ((retval = nc_def_dim(ncid, "kz", nkz, &kz_freq_dim)))
    ERR(retval);
  if ((retval = nc_def_dim(ncid, "t", NC_UNLIMITED, &time_freq_dim)))
    ERR(retval);
  if ((retval = nc_def_dim(ncid, "ri", 2, &ri_freq_dim)))
    ERR(retval);

  //Write title
  if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "title",strlen(title),title)))
    ERR(retval);

  //Define Variables
  kykx_mode_dim[1] = kykx_freq_dim;
  kykx_mode_dim[0] = nakyx_freq_dim;

  // NOTE: It appears the unlimited index must be the first index!
  freq_mode_dim[3] = ri_freq_dim;
  freq_mode_dim[2] = kz_freq_dim;
  freq_mode_dim[1] = nakyx_freq_dim;
  freq_mode_dim[0] = time_freq_dim;

  if ((retval = nc_def_var(ncid, "t", NC_FLOAT, 1,&time_freq_dim,&time_freq_id)))
    ERR(retval);
  if ((retval = nc_put_att_text(ncid, time_freq_id, "long_name", 4,"Time")))
    ERR(retval);
  if ((retval = nc_put_att_text(ncid, time_freq_id, "units", 4,"L/vt")))
    ERR(retval);
  if ((retval = nc_def_var(ncid, "kykx", NC_FLOAT, 2, kykx_mode_dim,&akyx_freq_id)))
    ERR(retval);
  if ((retval = nc_put_att_text(ncid, akyx_freq_id, "long_name", 12,"(ky, kx) rho")))
    ERR(retval);
  if ((retval = nc_def_var(ncid, "kz", NC_FLOAT, 1, &kz_freq_dim,&kz_freq_id)))
    ERR(retval);
  if ((retval = nc_def_var(ncid, "zp", NC_FLOAT, 4,freq_mode_dim, &zp_freq_id)))
    ERR(retval);
  if ((retval = nc_def_var(ncid, "zm", NC_FLOAT, 4,freq_mode_dim, &zm_freq_id)))
    ERR(retval);

  // End Definition Mode
  if ((retval = nc_enddef(ncid)))
    ERR(retval);


  if (dbg) printf("Definition Mode complete %s \n",filename);
 

  // Write values for variables kykx and kz
  // Create kykx  array and zero out
 //  create_2D_float_array(kykx,nk_freq,2);
  // if (dbg) printf("kykx[1][1]=%d \n",kykx[1][1]);
  kykx = (float *) malloc(nk_freq*2*sizeof(float));
  if (kykx==NULL)
    {
      fprintf(stderr, "Out of memory\n");
      exit(3);
    }
  for(i = 0; i < nk_freq*2; i++)
     kykx[i] = 0;

  // Create  allkz array and zero out
  allkz = (float *) malloc(nkz*sizeof(float));
  if (allkz==NULL)
    {
      fprintf(stderr, "Out of memory\n");
      exit(3);
    }
   for(i = 0; i < nkz; i++)
     allkz[i] = 0;

  if (dbg) printf("Created arrays for kykx and allkz \n");
  
  //  if (dbg) printf("sizeof(kykx[1][1])=%d \n",sizeof(kykx));

  //  if (dbg)
  // {
  //   for (i=0; i < nk_freq; i++) 
  //printf("kykx[%d][%d]=%f  kykx[%d][%d]=%f\n", i,0,kykx[i][0],i,1,kykx[i][1]);
  //  }
    
  


  //Set values for kykx and allkz
    for (inky = 0; inky < nky; inky++){
    for (inkx = 0; inkx < nkx; inkx++){
      ikykx=2*(inky*nkx+inkx);
      //      printf("inky= %d  inkx= %d  ind= %d, iky= %d  ikx= %d  ikykx= %d\n", inky,inkx,(inky)*nkx+inkx,inky_to_iky(inky),inkx_to_ikx(inkx),ikykx);
      kykx[ikykx]=ky(inky_to_iky(inky));
      kykx[ikykx+1]=kx(inkx_to_ikx(inkx)); 
    }
  }
  if (dbg) printf(" kykx values set \n");

   for (inkz=0; inkz<nkz; inkz++)
     allkz[inkz]=kz(inkz_to_ikz(inkz));
  if (dbg) printf(" allkz values set\n");
   
   // Put variable values into NetCDF file
     if ((retval = nc_put_var(ncid, akyx_freq_id,kykx)))
      ERR(retval);
   if ((retval = nc_put_var(ncid, kz_freq_id,allkz)))
     ERR(retval);

   if (dbg) printf("kykz and allkz written to NetCDF file set\n");

   //Free memory from kykx and allkz
   //   destroy_2D_float_array(kykx,nk_freq,2);
   free(kykx);
   free(allkz);

   //Sync to write to file
   if ((retval = nc_sync(ncid)))
     ERR(retval);
 
}


//////////////////////////////////////////////////////////////////////
// Close NetCDF file 
void finish_freq_diag(int ncid){
 
  int retval;
 
  if ((retval = nc_close(ncid)))
    ERR(retval);


}

//////////////////////////////////////////////////////////////////////
// Allocate 2D array (arrays of pointers, each pointing to array of pointers)
void create_2D_float_array(float **array, int nrow, int ncol){
  int i,j;
  
  array = (float **) malloc(nrow*sizeof(float *));
  if (array==NULL) {
    fprintf(stderr, "Out of memory\n");
    exit(3);
  }
  for (i=0; i<nrow; i++){
    array[i]=(float *) malloc(ncol*sizeof(float));
    if (array[i]==NULL)
      {
	fprintf(stderr, "Out of memory\n");
	exit(3);
      }
  }
  
  //  Zero out array
  for(i = 0; i < nrow; i++){
    for(j = 0; j < ncol; j++){
      array[i][j] = -99.;
      printf("array[%d][%d]= %f\n",i,j,array[i][j]);
    }
  }
  return;
}
//////////////////////////////////////////////////////////////////////
// Destroy 2D array (arrays of pointers, each pointing to array of pointers)
void destroy_2D_float_array(float **array, int nrow, int ncol)
{
  int i,j;

  for(i = 0; i < nrow; i++)
    free(array[i]);
  free(array);
}


//////////////////////////////////////////////////////////////////////
int inkx_to_ikx(int inkx)
{
  int nkx;
  // Compute total number of dealiased kx modes, nkx
  nkx=2*(Nx-1)/3+1;

  // Convert from dealiased index inkx to full index ikx
  if (inkx <= (Nx-1)/3){
    return inkx;
  } else {
    return (inkx-nkx);
  }
}
//////////////////////////////////////////////////////////////////////
int inky_to_iky(int inky)
{
  // Convert from dealiased index inky to full index iky
  return inky;
}
//////////////////////////////////////////////////////////////////////
int inkz_to_ikz(int inkz)
{
  int nkz;
  // Compute total number of dealiased kz modes, nkz
  nkz=2*(Nz-1)/3+1;

  // Convert from dealiased index inkz to full index ikz
  if (inkz <= (Nz-1)/3){
    return inkz;
  } else {
    return (inkz-nkz);
  }
}
